import * as AST8 from '@eslint-react/ast';
import { flip, birecord, dual, _, returnFalse } from '@eslint-react/eff';
import * as JSX4 from '@eslint-react/jsx';
import { AST_NODE_TYPES } from '@typescript-eslint/types';
import * as VAR from '@eslint-react/var';
import { unsafeDecodeSettings } from '@eslint-react/shared';

// src/component/component-collector.ts
function getElementNameAndRepresentName(node, context, polymorphicPropName, additionalComponents = []) {
  const name = JSX4.getElementName(node);
  if (name === name.toLowerCase()) return [name, name];
  const component = additionalComponents.findLast((c) => c.name === name || c.re.test(name));
  if (component != null) return [name, component.as];
  if (polymorphicPropName == null) return [name, name];
  const initialScope = context.sourceCode.getScope(node);
  const polymorphicProp = JSX4.findPropInAttributes(
    polymorphicPropName,
    initialScope,
    node.attributes
  );
  if (polymorphicProp == null) return [name, name];
  const polymorphicName = VAR.toResolved(JSX4.getPropValue(polymorphicProp, initialScope)).value;
  if (typeof polymorphicName === "string") {
    return [name, polymorphicName];
  }
  return [name, name];
}

// src/utils/get-id.ts
var id = 0n;
var getId = () => (id++).toString();
function isInitializedFromReact(name, initialScope, importSource = "react") {
  return name.toLowerCase() === "react" || VAR.isInitializedFromSource(
    name,
    importSource,
    initialScope
  );
}

// src/utils/is-from-react.ts
function isFromReact(name) {
  return (node, context) => {
    const settings = unsafeDecodeSettings(context.settings);
    if (!settings.strictImportCheck) {
      if (node.type === AST_NODE_TYPES.MemberExpression) {
        return node.object.type === AST_NODE_TYPES.Identifier && node.property.type === AST_NODE_TYPES.Identifier && node.property.name === name;
      }
      return node.name === name;
    }
    const initialScope = context.sourceCode.getScope(node);
    if (node.type === AST_NODE_TYPES.MemberExpression) {
      return node.object.type === AST_NODE_TYPES.Identifier && node.property.type === AST_NODE_TYPES.Identifier && node.property.name === name && isInitializedFromReact(node.object.name, initialScope, settings.importSource);
    }
    if (node.name === name) {
      return isInitializedFromReact(name, initialScope, settings.importSource);
    }
    return false;
  };
}
function isFromReactMember(memberName, name) {
  return (node, context) => {
    const settings = unsafeDecodeSettings(context.settings);
    if (!settings.strictImportCheck) {
      if (node.property.type !== AST_NODE_TYPES.Identifier || node.property.name !== name) {
        return false;
      }
      if (node.object.type === AST_NODE_TYPES.Identifier && node.object.name === memberName) {
        return true;
      }
      if (node.object.type === AST_NODE_TYPES.MemberExpression && node.object.object.type === AST_NODE_TYPES.Identifier && node.object.property.type === AST_NODE_TYPES.Identifier) {
        return node.object.property.name === memberName;
      }
      return false;
    }
    const initialScope = context.sourceCode.getScope(node);
    if (node.property.type !== AST_NODE_TYPES.Identifier || node.property.name !== name) {
      return false;
    }
    if (node.object.type === AST_NODE_TYPES.Identifier && node.object.name === memberName) {
      return isInitializedFromReact(node.object.name, initialScope, settings.importSource);
    }
    if (node.object.type === AST_NODE_TYPES.MemberExpression && node.object.object.type === AST_NODE_TYPES.Identifier && isInitializedFromReact(node.object.object.name, initialScope, settings.importSource) && node.object.property.type === AST_NODE_TYPES.Identifier) {
      return node.object.property.name === memberName;
    }
    return false;
  };
}
function isCallFromReact(name) {
  return dual(2, (node, context) => {
    if (node.type !== AST_NODE_TYPES.CallExpression) {
      return false;
    }
    if (!AST8.isOneOf([AST_NODE_TYPES.Identifier, AST_NODE_TYPES.MemberExpression])(node.callee)) {
      return false;
    }
    return isFromReact(name)(node.callee, context);
  });
}
function isCallFromReactMember(pragmaMemberName, name) {
  return dual(2, (node, context) => {
    if (node.type !== AST_NODE_TYPES.CallExpression) {
      return false;
    }
    if (!AST8.is(AST_NODE_TYPES.MemberExpression)(node.callee)) {
      return false;
    }
    return isFromReactMember(pragmaMemberName, name)(node.callee, context);
  });
}

// src/utils/is-react-api.ts
function isReactAPIWithName(name, member) {
  return member != null ? isFromReactMember(name, member) : isFromReact(name);
}
function isReactAPICallWithName(name, member) {
  return member != null ? isCallFromReactMember(name, member) : isCallFromReact(name);
}
var isChildrenCount = isReactAPIWithName("Children", "count");
var isChildrenForEach = isReactAPIWithName("Children", "forEach");
var isChildrenMap = isReactAPIWithName("Children", "map");
var isChildrenOnly = isReactAPIWithName("Children", "only");
var isChildrenToArray = isReactAPIWithName("Children", "toArray");
var isCloneElement = isReactAPIWithName("cloneElement");
var isCreateContext = isReactAPIWithName("createContext");
var isCreateElement = isReactAPIWithName("createElement");
var isCreateRef = isReactAPIWithName("createRef");
var isForwardRef = isReactAPIWithName("forwardRef");
var isMemo = isReactAPIWithName("memo");
var isChildrenCountCall = isReactAPICallWithName("Children", "count");
var isChildrenForEachCall = isReactAPICallWithName("Children", "forEach");
var isChildrenMapCall = isReactAPICallWithName("Children", "map");
var isChildrenOnlyCall = isReactAPICallWithName("Children", "only");
var isChildrenToArrayCall = isReactAPICallWithName("Children", "toArray");
var isCloneElementCall = isReactAPICallWithName("cloneElement");
var isCreateContextCall = isReactAPICallWithName("createContext");
var isCreateElementCall = isReactAPICallWithName("createElement");
var isCreateRefCall = isReactAPICallWithName("createRef");
var isForwardRefCall = isReactAPICallWithName("forwardRef");
var isMemoCall = isReactAPICallWithName("memo");

// src/element/hierarchy.ts
function isInsideCreateElementProps(node, context) {
  const call = AST8.findParentNode(node, isCreateElementCall(context));
  if (call == null) return false;
  const prop = AST8.findParentNode(node, AST8.is(AST_NODE_TYPES.ObjectExpression));
  if (prop == null) return false;
  return prop === call.arguments[1];
}
function isChildrenOfCreateElement(node, context) {
  const parent = node.parent;
  if (parent == null || parent.type !== AST_NODE_TYPES.CallExpression) return false;
  if (!isCreateElementCall(parent, context)) return false;
  return parent.arguments.slice(2).some((arg) => arg === node);
}

// src/hook/hook-name.ts
var RE_HOOK_NAME = /^use[A-Z\d]/u;
function isReactHookName(name) {
  return name === "use" || RE_HOOK_NAME.test(name);
}

// src/hook/is.ts
function isReactHook(node) {
  if (node == null) return _;
  const id2 = AST8.getFunctionIdentifier(node);
  return id2?.name != null && isReactHookName(id2.name);
}
function isReactHookCall(node) {
  if (node == null) return false;
  if (node.type !== AST_NODE_TYPES.CallExpression) {
    return false;
  }
  if (node.callee.type === AST_NODE_TYPES.Identifier) {
    return isReactHookName(node.callee.name);
  }
  if (node.callee.type === AST_NODE_TYPES.MemberExpression) {
    return node.callee.property.type === AST_NODE_TYPES.Identifier && isReactHookName(node.callee.property.name);
  }
  return false;
}
function isReactHookCallWithName(node, context) {
  if (node == null) return returnFalse;
  const settings = unsafeDecodeSettings(context.settings);
  return (name) => {
    const initialScope = context.sourceCode.getScope(node);
    switch (true) {
      case (node.callee.type === AST_NODE_TYPES.Identifier && node.callee.name === name):
        return !settings.strictImportCheck || isInitializedFromReact(name, initialScope, settings.importSource);
      case (node.callee.type === AST_NODE_TYPES.MemberExpression && node.callee.property.type === AST_NODE_TYPES.Identifier && node.callee.property.name === name && "name" in node.callee.object):
        return !settings.strictImportCheck || isInitializedFromReact(node.callee.object.name, initialScope, settings.importSource);
      default:
        return false;
    }
  };
}
function isReactHookCallWithNameLoose(node) {
  if (node == null) return returnFalse;
  return (name) => {
    switch (node.callee.type) {
      case AST_NODE_TYPES.Identifier:
        return node.callee.name === name;
      case AST_NODE_TYPES.MemberExpression:
        return node.callee.property.type === AST_NODE_TYPES.Identifier && node.callee.property.name === name;
      default:
        return false;
    }
  };
}
function isReactHookCallWithNameAlias(name, context, alias) {
  const settings = unsafeDecodeSettings(context);
  return (node) => {
    const initialScope = context.sourceCode.getScope(node);
    switch (true) {
      case (node.callee.type === AST_NODE_TYPES.Identifier && node.callee.name === name):
        return !settings.strictImportCheck || isInitializedFromReact(name, initialScope, settings.importSource);
      case (node.callee.type === AST_NODE_TYPES.MemberExpression && node.callee.property.type === AST_NODE_TYPES.Identifier && node.callee.property.name === name && "name" in node.callee.object):
        return !settings.strictImportCheck || isInitializedFromReact(node.callee.object.name, initialScope, settings.importSource);
      default:
        return alias.some(isReactHookCallWithNameLoose(node));
    }
  };
}
function isUseEffectCallLoose(node) {
  if (node == null) return false;
  if (node.type !== AST_NODE_TYPES.CallExpression) {
    return false;
  }
  switch (node.callee.type) {
    case AST_NODE_TYPES.Identifier:
      return /^use\w*Effect$/u.test(node.callee.name);
    case AST_NODE_TYPES.MemberExpression:
      return node.callee.property.type === AST_NODE_TYPES.Identifier && /^use\w*Effect$/u.test(node.callee.property.name);
    default:
      return false;
  }
}
var isUseCallbackCall = flip(isReactHookCallWithName)("useCallback");
var isUseContextCall = flip(isReactHookCallWithName)("useContext");
var isUseDebugValueCall = flip(isReactHookCallWithName)("useDebugValue");
var isUseDeferredValueCall = flip(isReactHookCallWithName)("useDeferredValue");
var isUseEffectCall = flip(isReactHookCallWithName)("useEffect");
var isUseIdCall = flip(isReactHookCallWithName)("useId");
var isUseImperativeHandleCall = flip(isReactHookCallWithName)("useImperativeHandle");
var isUseInsertionEffectCall = flip(isReactHookCallWithName)("useInsertionEffect");
var isUseLayoutEffectCall = flip(isReactHookCallWithName)("useLayoutEffect");
var isUseMemoCall = flip(isReactHookCallWithName)("useMemo");
var isUseReducerCall = flip(isReactHookCallWithName)("useReducer");
var isUseRefCall = flip(isReactHookCallWithName)("useRef");
var isUseStateCall = flip(isReactHookCallWithName)("useState");
var isUseSyncExternalStoreCall = flip(isReactHookCallWithName)("useSyncExternalStore");
var isUseTransitionCall = flip(isReactHookCallWithName)("useTransition");

// src/hook/hierarchy.ts
function isInsideReactHook(node) {
  return isReactHook(AST8.findParentNode(node, AST8.isFunction));
}
function useHookCollector() {
  const hooks = /* @__PURE__ */ new Map();
  const fEntries = [];
  const onFunctionEnter = (node) => {
    const id2 = AST8.getFunctionIdentifier(node);
    const name = id2?.name;
    if (name != null && isReactHookName(name)) {
      const key = getId();
      fEntries.push({ key, node });
      hooks.set(key, {
        id: id2,
        key,
        kind: "function",
        name,
        node,
        flag: 0n,
        hint: 0n,
        hookCalls: []
      });
      return;
    }
    fEntries.push({ key: _, node });
  };
  const onFunctionExit = () => {
    fEntries.pop();
  };
  const ctx = {
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    getAllHooks(node) {
      return hooks;
    },
    getCurrentHooks() {
      return new Map(hooks);
    }
  };
  const listeners = {
    ":function[type]": onFunctionEnter,
    ":function[type]:exit": onFunctionExit,
    "CallExpression[type]"(node) {
      if (!isReactHookCall(node)) {
        return;
      }
      const fEntry = fEntries.at(-1);
      if (fEntry?.key == null) {
        return;
      }
      const hook = hooks.get(fEntry.key);
      if (hook == null) {
        return;
      }
      hook.hookCalls.push(node);
    }
  };
  return { ctx, listeners };
}
var ERComponentHint = {
  ...JSX4.JSXValueHint,
  // 1n << 0n - 1n << 63n are reserved for JSXValueHint
  // Skip function component created by React.memo
  SkipMemo: 1n << 64n,
  // Skip function component created by React.forwardRef
  SkipForwardRef: 1n << 65n,
  // Skip function component defined in map function callback
  SkipMapCallback: 1n << 66n,
  // Skip function component defined on object method
  SkipObjectMethod: 1n << 67n,
  // Skip function component defined on class method
  SkipClassMethod: 1n << 68n,
  // Skip function component defined on class property
  SkipClassProperty: 1n << 69n
};
var DEFAULT_COMPONENT_HINT = 0n | ERComponentHint.SkipBooleanLiteral | ERComponentHint.SkipMapCallback | ERComponentHint.SkipNumberLiteral | ERComponentHint.SkipStringLiteral | ERComponentHint.SkipUndefined | ERComponentHint.SkipEmptyArray | ERComponentHint.StrictArray | ERComponentHint.StrictConditional | ERComponentHint.StrictLogical;

// src/component/component-flag.ts
var ERClassComponentFlag = {
  None: 0n,
  PureComponent: 1n << 0n
  // Reserved for future use
  // CreateElement: 1n << 1n,
};
var ERFunctionComponentFlag = {
  None: 0n,
  Memo: 1n << 0n,
  ForwardRef: 1n << 1n
  // Reserved for future use
  // CreateElement: 1n << 2n,
  // Reserved for future use
  // hasHooks: 1n << 3n,
  // Reserved for future use
  // Async: 1n << 4n,
};
function isComponentWrapperCall(node, context) {
  if (node.type !== AST_NODE_TYPES.CallExpression) {
    return false;
  }
  return isMemoCall(node, context) || isForwardRefCall(node, context) || isReactHookCallWithNameLoose(node)("useCallback");
}
function getFunctionComponentIdentifier(node, context) {
  const functionId = AST8.getFunctionIdentifier(node);
  if (functionId != null) {
    return functionId;
  }
  const { parent } = node;
  if (parent.type === AST_NODE_TYPES.CallExpression && isComponentWrapperCall(parent, context) && parent.parent.type === AST_NODE_TYPES.VariableDeclarator && parent.parent.id.type === AST_NODE_TYPES.Identifier) {
    return parent.parent.id;
  }
  if (parent.type === AST_NODE_TYPES.CallExpression && isComponentWrapperCall(parent, context) && parent.parent.type === AST_NODE_TYPES.CallExpression && isComponentWrapperCall(parent.parent, context) && parent.parent.parent.type === AST_NODE_TYPES.VariableDeclarator && parent.parent.parent.id.type === AST_NODE_TYPES.Identifier) {
    return parent.parent.parent.id;
  }
  return _;
}
var RE_COMPONENT_NAME = /^_?[A-Z]/u;
function getComponentNameFromIdentifier(node) {
  if (node == null) return _;
  return Array.isArray(node) ? node.map((n) => n.name).join(".") : node.name;
}
function isComponentName(name) {
  return RE_COMPONENT_NAME.test(name);
}
function isClassComponent(node) {
  if ("superClass" in node && node.superClass != null) {
    const re = /^(?:Pure)?Component$/u;
    switch (true) {
      case node.superClass.type === AST_NODE_TYPES.Identifier:
        return re.test(node.superClass.name);
      case (node.superClass.type === AST_NODE_TYPES.MemberExpression && node.superClass.property.type === AST_NODE_TYPES.Identifier):
        return re.test(node.superClass.property.name);
    }
  }
  return false;
}
function isPureComponent(node) {
  if ("superClass" in node && node.superClass != null) {
    const re = /^PureComponent$/u;
    switch (true) {
      case node.superClass.type === AST_NODE_TYPES.Identifier:
        return re.test(node.superClass.name);
      case (node.superClass.type === AST_NODE_TYPES.MemberExpression && node.superClass.property.type === AST_NODE_TYPES.Identifier):
        return re.test(node.superClass.property.name);
    }
  }
  return false;
}

// src/component/component-render-method.ts
function isRenderMethodLike(node) {
  return (node.type === AST_NODE_TYPES.MethodDefinition || node.type === AST_NODE_TYPES.PropertyDefinition) && node.key.type === AST_NODE_TYPES.Identifier && node.key.name === "render" && node.parent.parent.type === AST_NODE_TYPES.ClassDeclaration;
}
function isFunctionOfRenderMethod(node) {
  if (!isRenderMethodLike(node.parent)) {
    return false;
  }
  return isClassComponent(node.parent.parent.parent);
}
function isInsideRenderMethod(node) {
  return AST8.findParentNode(node, (node2) => isRenderMethodLike(node2) && isClassComponent(node2.parent.parent)) != null;
}

// src/component/misc.ts
function hasNoneOrValidComponentName(node, context) {
  const id2 = getFunctionComponentIdentifier(node, context);
  if (id2 == null) return true;
  const name = Array.isArray(id2) ? id2.at(-1)?.name : id2.name;
  return name != null && isComponentName(name);
}

// src/component/component-collector.ts
function hasValidHierarchy(node, context, hint) {
  if (isChildrenOfCreateElement(node, context) || isFunctionOfRenderMethod(node)) {
    return false;
  }
  if (hint & ERComponentHint.SkipMapCallback && AST8.isMapCallLoose(node.parent)) {
    return false;
  }
  if (hint & ERComponentHint.SkipObjectMethod && AST8.isFunctionOfObjectMethod(node.parent)) {
    return false;
  }
  if (hint & ERComponentHint.SkipClassMethod && AST8.isFunctionOfClassMethod(node.parent)) {
    return false;
  }
  if (hint & ERComponentHint.SkipClassProperty && AST8.isFunctionOfClassProperty(node.parent)) {
    return false;
  }
  const boundaryNode = AST8.findParentNode(
    node,
    AST8.isOneOf([
      AST_NODE_TYPES.JSXExpressionContainer,
      AST_NODE_TYPES.ArrowFunctionExpression,
      AST_NODE_TYPES.FunctionExpression,
      AST_NODE_TYPES.Property,
      AST_NODE_TYPES.ClassBody
    ])
  );
  return boundaryNode == null || boundaryNode.type !== AST_NODE_TYPES.JSXExpressionContainer;
}
function getComponentFlag(initPath) {
  let flag = ERFunctionComponentFlag.None;
  if (initPath != null && AST8.hasCallInFunctionInitPath("memo", initPath)) {
    flag |= ERFunctionComponentFlag.Memo;
  }
  if (initPath != null && AST8.hasCallInFunctionInitPath("forwardRef", initPath)) {
    flag |= ERFunctionComponentFlag.ForwardRef;
  }
  return flag;
}
var displayNameAssignmentSelector = "AssignmentExpression[type][operator='='][left.type='MemberExpression'][left.property.name='displayName']";
function useComponentCollector(context, hint = DEFAULT_COMPONENT_HINT, options = {}) {
  const { collectDisplayName = false, collectHookCalls = false } = options;
  const jsxCtx = { getScope: (node) => context.sourceCode.getScope(node) };
  const components = /* @__PURE__ */ new Map();
  const functionEntries = [];
  const getCurrentEntry = () => functionEntries.at(-1);
  const onFunctionEnter = (node) => {
    const key = getId();
    functionEntries.push({ key, node, hookCalls: [], isComponent: false });
  };
  const onFunctionExit = () => {
    const entry = functionEntries.at(-1);
    if (entry == null) return;
    if (!entry.isComponent) {
      return functionEntries.pop();
    }
    const shouldDrop = AST8.getNestedReturnStatements(entry.node.body).slice().reverse().some((r) => {
      return context.sourceCode.getScope(r).block === entry.node && r.argument != null && !JSX4.isJSXValue(r.argument, jsxCtx, hint);
    });
    if (shouldDrop) {
      components.delete(entry.key);
    }
    return functionEntries.pop();
  };
  const ctx = {
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    getAllComponents(node) {
      return components;
    },
    getCurrentEntries() {
      return [...functionEntries];
    },
    getCurrentEntry
  };
  const listeners = {
    ":function[type]": onFunctionEnter,
    ":function[type]:exit": onFunctionExit,
    "ArrowFunctionExpression[type][body.type!='BlockStatement']"() {
      const entry = getCurrentEntry();
      if (entry == null) {
        return;
      }
      const { body } = entry.node;
      const isComponent = hasNoneOrValidComponentName(entry.node, context) && JSX4.isJSXValue(body, jsxCtx, hint) && hasValidHierarchy(entry.node, context, hint);
      if (!isComponent) {
        return;
      }
      const initPath = AST8.getFunctionInitPath(entry.node);
      const id2 = getFunctionComponentIdentifier(entry.node, context);
      const name = getComponentNameFromIdentifier(id2);
      const key = getId();
      components.set(key, {
        id: id2,
        key,
        kind: "function",
        name,
        node: entry.node,
        displayName: _,
        flag: getComponentFlag(initPath),
        hint,
        hookCalls: entry.hookCalls,
        initPath
      });
    },
    ...collectDisplayName ? {
      [displayNameAssignmentSelector](node) {
        const { left, right } = node;
        const componentName = left.object.type === AST_NODE_TYPES.Identifier ? left.object.name : _;
        const component = [...components.values()].findLast(({ name }) => name != null && name === componentName);
        if (component == null) {
          return;
        }
        component.displayName = right;
      }
    } : {},
    ...collectHookCalls ? {
      "CallExpression[type]:exit"(node) {
        if (!isReactHookCall(node)) {
          return;
        }
        const entry = getCurrentEntry();
        if (entry == null) {
          return;
        }
        entry.hookCalls.push(node);
      }
    } : {},
    "ReturnStatement[type]"(node) {
      const entry = getCurrentEntry();
      if (entry == null) {
        return;
      }
      const isComponent = hasNoneOrValidComponentName(entry.node, context) && JSX4.isJSXValue(node.argument, jsxCtx, hint) && hasValidHierarchy(entry.node, context, hint);
      if (!isComponent) {
        return;
      }
      functionEntries.pop();
      functionEntries.push({ ...entry, isComponent });
      const initPath = AST8.getFunctionInitPath(entry.node);
      const id2 = getFunctionComponentIdentifier(entry.node, context);
      const name = getComponentNameFromIdentifier(id2);
      components.set(entry.key, {
        id: id2,
        key: entry.key,
        kind: "function",
        name,
        node: entry.node,
        displayName: _,
        flag: getComponentFlag(initPath),
        hint,
        hookCalls: entry.hookCalls,
        initPath
      });
    }
  };
  return { ctx, listeners };
}
function useComponentCollectorLegacy() {
  const components = /* @__PURE__ */ new Map();
  const ctx = {
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    getAllComponents(node) {
      return components;
    }
  };
  const collect = (node) => {
    if (!isClassComponent(node)) {
      return;
    }
    const id2 = AST8.getClassIdentifier(node);
    const key = getId();
    const flag = isPureComponent(node) ? ERClassComponentFlag.PureComponent : ERClassComponentFlag.None;
    components.set(
      key,
      {
        id: id2,
        key,
        kind: "class",
        name: id2?.name,
        node,
        // TODO: Get displayName of class component
        displayName: _,
        flag,
        hint: 0n,
        // TODO: Get methods of class component
        methods: []
      }
    );
  };
  const listeners = {
    "ClassDeclaration[type]": collect,
    "ClassExpression[type]": collect
  };
  return { ctx, listeners };
}
function isSetupFunction(node) {
  if (node == null) return _;
  return node.parent?.type === AST_NODE_TYPES.CallExpression && node.parent.callee !== node && node.parent.callee.type === AST_NODE_TYPES.Identifier && node.parent.arguments.at(0) === node && isUseEffectCallLoose(node.parent);
}
function isCleanupFunction(node) {
  const nearReturn = AST8.findParentNode(node, AST8.is(AST_NODE_TYPES.ReturnStatement));
  const nearFunction = AST8.findParentNode(node, AST8.isFunction);
  const nearFunctionOfReturn = AST8.findParentNode(nearReturn, AST8.isFunction);
  return nearFunction === nearFunctionOfReturn && isSetupFunction(nearFunction);
}
function isComponentDidMount(node) {
  return AST8.isOneOf([AST_NODE_TYPES.MethodDefinition, AST_NODE_TYPES.PropertyDefinition])(node) && node.key.type === AST_NODE_TYPES.Identifier && node.key.name === "componentDidMount";
}
function isComponentWillUnmount(node) {
  return AST8.isOneOf([AST_NODE_TYPES.MethodDefinition, AST_NODE_TYPES.PropertyDefinition])(node) && node.key.type === AST_NODE_TYPES.Identifier && node.key.name === "componentWillUnmount";
}
function isComponentDidMountFunction(node) {
  return AST8.isFunction(node) && isComponentDidMount(node.parent) && node.parent.value === node;
}
function isComponentWillUnmountFunction(node) {
  return AST8.isFunction(node) && isComponentWillUnmount(node.parent) && node.parent.value === node;
}
var ERPhaseRelevance = birecord({
  mount: "unmount",
  setup: "cleanup"
});
var isInversePhase = dual(2, (a, b) => ERPhaseRelevance.get(a) === b);
function isDirectValueOfRenderPropertyLoose(node) {
  const matching = (node2) => {
    return node2.type === AST_NODE_TYPES.Property && node2.key.type === AST_NODE_TYPES.Identifier && node2.key.name.startsWith("render");
  };
  return matching(node) || node.parent != null && matching(node.parent);
}
function isDeclaredInRenderPropLoose(node) {
  if (isDirectValueOfRenderPropertyLoose(node)) {
    return true;
  }
  const parent = AST8.findParentNode(node, AST8.is(AST_NODE_TYPES.JSXExpressionContainer))?.parent;
  if (parent?.type !== AST_NODE_TYPES.JSXAttribute) {
    return false;
  }
  return parent.name.type === AST_NODE_TYPES.JSXIdentifier && parent.name.name.startsWith("render");
}
function isRenderFunctionLoose(node, context) {
  const { body, parent } = node;
  if (AST8.getFunctionIdentifier(node)?.name.startsWith("render")) {
    return parent.type === AST_NODE_TYPES.JSXExpressionContainer && parent.parent.type === AST_NODE_TYPES.JSXAttribute && parent.parent.name.type === AST_NODE_TYPES.JSXIdentifier && parent.parent.name.name.startsWith("render");
  }
  return JSX4.isJSXValue(
    body,
    {
      getScope: (node2) => context.sourceCode.getScope(node2)
    },
    JSX4.JSXValueHint.SkipNullLiteral | JSX4.JSXValueHint.SkipUndefined | JSX4.JSXValueHint.StrictLogical | JSX4.JSXValueHint.StrictConditional
  );
}
function isRenderPropLoose(node, context) {
  if (node.name.type !== AST_NODE_TYPES.JSXIdentifier) {
    return false;
  }
  return node.name.name.startsWith("render") && node.value?.type === AST_NODE_TYPES.JSXExpressionContainer && AST8.isFunction(node.value.expression) && isRenderFunctionLoose(node.value.expression, context);
}
function isThisSetState(node) {
  const { callee } = node;
  return callee.type === AST_NODE_TYPES.MemberExpression && AST8.isThisExpression(callee.object) && callee.property.type === AST_NODE_TYPES.Identifier && callee.property.name === "setState";
}

export { DEFAULT_COMPONENT_HINT, ERClassComponentFlag, ERComponentHint, ERFunctionComponentFlag, ERPhaseRelevance, RE_COMPONENT_NAME, RE_HOOK_NAME, getComponentNameFromIdentifier, getElementNameAndRepresentName, getFunctionComponentIdentifier, getId, hasNoneOrValidComponentName, isCallFromReact, isCallFromReactMember, isChildrenCount, isChildrenCountCall, isChildrenForEach, isChildrenForEachCall, isChildrenMap, isChildrenMapCall, isChildrenOfCreateElement, isChildrenOnly, isChildrenOnlyCall, isChildrenToArray, isChildrenToArrayCall, isClassComponent, isCleanupFunction, isCloneElement, isCloneElementCall, isComponentDidMount, isComponentDidMountFunction, isComponentName, isComponentWillUnmount, isComponentWillUnmountFunction, isCreateContext, isCreateContextCall, isCreateElement, isCreateElementCall, isCreateRef, isCreateRefCall, isDeclaredInRenderPropLoose, isDirectValueOfRenderPropertyLoose, isForwardRef, isForwardRefCall, isFromReact, isFromReactMember, isFunctionOfRenderMethod, isInitializedFromReact, isInsideCreateElementProps, isInsideReactHook, isInsideRenderMethod, isInversePhase, isMemo, isMemoCall, isPureComponent, isReactAPICallWithName, isReactAPIWithName, isReactHook, isReactHookCall, isReactHookCallWithName, isReactHookCallWithNameAlias, isReactHookCallWithNameLoose, isReactHookName, isRenderFunctionLoose, isRenderMethodLike, isRenderPropLoose, isSetupFunction, isThisSetState, isUseCallbackCall, isUseContextCall, isUseDebugValueCall, isUseDeferredValueCall, isUseEffectCall, isUseEffectCallLoose, isUseIdCall, isUseImperativeHandleCall, isUseInsertionEffectCall, isUseLayoutEffectCall, isUseMemoCall, isUseReducerCall, isUseRefCall, isUseStateCall, isUseSyncExternalStoreCall, isUseTransitionCall, useComponentCollector, useComponentCollectorLegacy, useHookCollector };
